#! /usr/bin/env python
#coding=utf-8

'''
vdrswitchoff.py written by Wacholder

Bescheibung:
Die Idee ist, dieses Skript als Cronjob zu starten.
Es wird über einen Eintrag in der crontab zu fest definierten Zeiten
ausgeführt.
Über das vdr-plugin-osdserver wird ein countdown an den TV ausgegeben.
Die Dauer der Meldung ist einstellbar.
Innerhalb der Meldung muß der User die OK Taste der FB drücken, ansonsten
wird per SVDRP das Herunterfahren des VDR ausgelöst (bei Anwendung des
Parameters "--shutdown").
Unabhängig davon ob der VDR gerade aufnimmt kann ein weiterer Befehl ausgelöst
werden (hier Ausschalten des TV).

Wird vdrswitchoff ohne Parameter gestartet, so wird der countdown gestartet,
bei Useraktivität wird das Skript mit "exit 1" beendet. Somit kann es auch aus
zusätzliche Abfrage in den VDR shutdown-hooks verwendet werden.

Installation:
Benötigt wird der VDR OSDServer für die Ausgabe. Ansonsten das Skript an
beliebiger Stelle schieben und ausführbar machen. Der Konfigurationbereich
des Skriptes muß noch auf die aktuelle Umgebung angepasst werden.
'''

#Konfiguration
msgduration = 60            #Dauer des Countdown
IPAdress = 'localhost'      #IP des VDR
OSDServerPort = 2010        #Port des OSDServers
SVDRPPort = 6419            #Port des SVDRP
shell_cmd = "sispmctl"      #auszuführender Befehl
shell_param = "-f 1"        #Parameter für shell_cmd

#Code:
import socket, sys, subprocess
from syslog import *

__name__ = "VDRswitchoff 0.1"


class SVDRP(object):

    def __init__(self, host='localhost', port=2001):
        self._addr = (host, port)

    def readline(self, sock):
        line = ''
        while 1:
            c = sock.recv(1)
            line += c
            if c == '' or c == '\n':
                break
        return line

    def connect(self):
        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        sock.connect(self._addr)
        welcome = self.readline(sock)
        code, hostname, proto, app = welcome.split(' ')[:4]
        if not (code == '220' and proto == 'SVDRP' and
                                                app == 'VideoDiskRecorder'):
            print "Not talking to VDR!"
            print welcome
            sock.close()
            raise ValueError
        return sock

    def disconnect(self, sock):
        sock.sendall('QUIT\n')
        quit = self.readline(sock)
        sock.close()

    def execute(self, command):
        sock = self.connect()
        if not command.endswith('\n'):
            command += '\n'
        sock.sendall(command)
        buffer = ''
        while 1:
            line = self.readline(sock)
            buffer += line
            if line == '' or line[3] == ' ':
                break
        self.disconnect(sock)
        return buffer


class OSD(object):

    def __init__(self, host='localhost', port=2010):
        self._addr = (host, port)
        self.sock = None

    def readline(self):
        line = ''
        while 1:
            c = self.sock.recv(1)
            line += c
            if c == '' or c == '\n':
                break
        return line

    def sendCmd(self, command):
        if not command.endswith('\n'):
            command += '\n'
        self.sock.sendall(command)
        return self.readline()

    def connect(self):
        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.sock = sock
        sock.connect(self._addr)
        buffer = ''
        while 1:
            line = self.readline()
            buffer += line
            if line == '' or line[3] == ' ':
                break
        if not buffer[:3] == '201':
            print "connection not successfull", buffer
            sock.close()
            raise ValueError
        else:
            self.sendCmd("Version 0.1\n")
        return sock, buffer

    def disconnect(self):
        quit = self.sendCmd('QUIT')
        self.sock.close()
        return quit


def shutdownVDR(host=IPAdress, port=2004):
    #initialisiert das Herunterfahren des VDR
    svdr = SVDRP(host, port)
    svdr.connect()
    buffer = svdr.execute('HITK Power')
    if buffer[:3] == "250":
        syslog(LOG_INFO, "'SVDRP HITK Power' passed")
        return True
    else:
        syslog(LOG_INFO, "'SVDRP HITK Power' failed")
        return False


def switchoffTV():
    #Schaltet die Steckdose aus
    shell_ausgabe = subprocess.Popen([shell_cmd, shell_param],
                                stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    stdout, stderr = shell_ausgabe.communicate()
    syslog(LOG_INFO, "switch-off TV: " + stdout+stderr)

openlog(__name__, LOG_PID|LOG_CONS, LOG_LOCAL3)
s = OSD(IPAdress, OSDServerPort)
s.connect()[1]
s.sendCmd("Meldung = NEW MENU '%s'"%__name__)
s.sendCmd("txt1 = Meldung.AddNew OsdItem -unselectable ' VDR wird in %ssek herunterfahren !'" %msgduration)
s.sendCmd("txt2 = Meldung.AddNew OsdItem -unselectable 'Drücke OK um das Herunterfahren abzubrechen'")
s.sendCmd("Meldung.EnableEvent keyOk")
s.sendCmd("Meldung.show \n")
for step in range(msgduration, 0, -1):
    #Schleife für countdown
    s.sendCmd("txt1.SETTEXT 'VDR wird in %ssek herunterfahren !'" %step)
    s.sendCmd("Meldung.show \n")
    event = (s.sendCmd("Meldung.Sleepevent -timeout 1 "))
    s.readline()
    if event[:3] == "300":
        #OK auf der FB wurde gedrückt
        s.sendCmd("txt1.SETTEXT 'VDR BLEIBT AN !'")
        s.sendCmd("txt2.delete")
        s.sendCmd("Meldung.show \n")
        s.sendCmd("Meldung.SENDSTATE osEND \n")
        s.sendCmd("Meldung.DELETE")
        syslog(LOG_INFO, "VDR Poweroff was aborted by user")
        sys.exit("VDR Poweroff was aborted by user")
        break
try:
    if sys.argv[1] == '--shutdown':
        syslog(LOG_INFO, "started with '--shutdown'")
        if not shutdownVDR(IPAdress, SVDRPPort):
            syslog(LOG_INFO, 'starting shutdown failed !')
            s.sendCmd("MESSAGE -error -seconds 5 'starting shutdown failed !'\n")
except:
    pass

switchoffTV()
